#!/usr/bin/python
# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4

import ldap
import ldif
import ldap.modlist
import ConfigParser
import os
import os.path
import sys
import datetime
import time
import re
from string import Template

outfile=sys.stdout


def is_valid_mac_address(mac):
    if mac is None or not re.match("^([0-9A-Fa-f]{2}[:-]){5}([0-9A-Fa-f]{2})$",mac):
        return False
    return True


def write_dhcp_config(clients):
    
    f = open("dhcpd.hosts.conf","w")

    for name,ip,mac,network_name in clients:
        if not is_valid_mac_address(mac):
            continue

        f.write("        host %s {\n" % name)
        f.write("            hardware ethernet %s;\n" % mac)
        if ip!=None and not ip.startswith("127."):
                f.write("            fixed-address %s;\n" % ip)
                includepath = "/etc/dhcp/dhcpd.%s.conf" % name
                if os.path.isfile(includepath):
                    f.write("            include \"%s\";\n" % includepath)
        f.write("        }\n\n")
    f.close()

def get_zone_name(zones,name):
    i = name.rfind('.')
    if i>-1:
        zone = name[i+1:]
    else:
        zone = name
    if zone in zones:
        return zone
    else:
        return zones[0]

def add_zone(zones,name):
    zone = get_zone_name(name)
    if not zone in zones:
        zones.append(zone)

def get_zone(zones,name):
    zone = get_zone_name(name)
    if zone in zones:
        return zone
    else:
        return None

def get_network_name(baseDn, seeAlso):
    for a in seeAlso:
        if a.endswith(baseDn):
            i = a.find(',')
            if i>-1:
                rdn = a[:i]
                i = rdn.find('=')
                if i>-1:
                    name = rdn[i+1:].strip()
                    return name
    return None

def write_template(outfile,template_path,params):
    template_file = open(template_path,"r")
    template = Template(template_file.read())
    template_file.close()

    outfile.write(template.substitute(params))

def write_zone_header(zonefile,zonename,origin=None):
    if origin == None:
        origin = zonename

    # serial = datetime.datetime.now().strftime("%Y%m%d%H")
    serial = str(int(time.time()))
    params = { "zone" : zonename, "serial" : serial, "origin" : origin }
    write_template(zonefile,"/opt/dass-it/etc/zone.header.template",params)

def write_zonefiles(clients,zones,primaryzone,hidden_domains):
    # print clients

    is_first = True
    print primaryzone

    for z in zones:
        print "--------------------------",z,"-------------------------"
        f = open("db.%s" % z,"w")
        write_zone_header(f,z)

        for client_name, ip, mac, client_network in clients:
            client_zone = get_zone_name(zones,client_network)
            if client_zone == z or (is_first and client_network == primaryzone):
                if client_network.endswith("."+z):
                    client_network = client_network[:client_network.rfind(".")]

                if client_network in hidden_domains:
                    client_network=primaryzone

                if ip[0].isalpha() :
                    t = "CNAME"
                    if client_network != primaryzone:
                        ip += '.' + client_network
                else:
                    t = "A"

                if client_network == primaryzone:
                    f.write("%s\t\tIN %s %s\n" % (client_name, t, ip))
                else:
                    f.write("%s.%s\t\tIN %s %s\n" % (client_name, client_network, t, ip))
        is_first = False

        f.close()

def write_rev_zonefile(path,clients,revzone,zones,primaryzone,primarydomain,hidden_domains,filter=None):
    f = open(path,"w")
    write_zone_header(f,revzone,revzone+".in-addr.arpa")
    for name,ip,mac,domain in clients:
        if ip==None:
                continue
        if filter!=None and not ip.startswith(filter):
                continue

        if domain.endswith(primaryzone):
            domain = domain[:-(len(primaryzone)+1)] + primarydomain

        if domain in hidden_domains:
            domain = primarydomain

        absolute_zone = False
        for z in zones:
            if domain.endswith(z):
                absolute_zone = True
                break

        if not absolute_zone:
            domain = domain + "." + primarydomain

        a=reversed(ip.split('.')[1:4])
        hostpart='.'.join(a)
        f.write("%-20s        IN PTR %s.%s.\n" % (hostpart,name,domain))
    f.close()

def sort_aliases(dn, names):
    aliases = names
    name = None
    for n in names:
        if dn.startswith('cn=%s,' % n):
            name = n
            aliases.remove(n)

    return name, aliases

if __name__ == "__main__":
    conffile="/opt/dass-it/etc/ldap2dnsdhcp.conf"

    config=ConfigParser.ConfigParser()
    config.read(conffile)
   
    nodename = config.get("global","nodename")
 
    server = config.get("ldap","server") 
    admindn = config.get("ldap","bindDn") 
    adminpw = config.get("ldap","bindPassword")
    basedn = config.get("ldap","baseDn")
    filter = config.get("ldap","filter")

    computersdn = config.get("computers","baseDn")
    devicesdn = config.get("devices","baseDn")
  
    dnsdir = config.get("dns","directory")
    dnsdomain = config.get("dns","domain")
    dnsprimary = config.get("dns","primary")
    dnsadmin = config.get("dns","rname")

    hidden_domains = config.get("dns","hide_subdomains")

    dnsrevzone = config.get("dns","reverseZone")
    filialserverIpLastOctet = config.get("dns","serverLastOctet")
    dhcpdir = config.get("dhcp","directory")
 
    networksDn  = basedn

    con = ldap.open(server,port=389)
    con.start_tls_s()
    con.simple_bind_s(admindn,adminpw)

    zones = [ x.strip() for x in dnsdomain.split(',') ]
    hidden_domains = [ x.strip() for x in hidden_domains.split(',') ]

    networks=[]
    clients=[]
    results=con.search_s(networksDn,ldap.SCOPE_SUBTREE,"objectClass=ipNetwork",None)
    for r in results:
        dn,attrs=r
        networks.append((attrs['cn'][0],attrs['ipNetworkNumber'][0],attrs['ipNetmaskNumber'][0]))
        ip=attrs['ipNetworkNumber'][0]
        # ip='.'.join(ip.split('.')[0:3])+'.'+filialserverIpLastOctet
        # clients.append((attrs['cn'][0],ip,None))

    results = con.search_s(computersdn,ldap.SCOPE_SUBTREE, "objectClass=ipHost",None)
    for dn,attrs in results:
        ip = attrs['ipHostNumber'][0]
        name = attrs['cn'][0]
        network_name = get_network_name(networksDn, attrs['seeAlso']) 

        if len(attrs['cn'])>1:
            name, aliases = sort_aliases(dn, attrs['cn'])
            for a in aliases:
                print name,"alias",a
                clients.append((a,name,None,network_name))
        if attrs.has_key('macAddress'):
            mac = attrs['macAddress'][0]
        else:
            mac = None
        clients.append((name,ip,mac,network_name))

    results = con.search_s(devicesdn,ldap.SCOPE_SUBTREE, "objectClass=ipHost",None)
    for dn,attrs in results:
        ip = attrs['ipHostNumber'][0]
        name = attrs['cn'][0]
        network_name = attrs['ou'][0]

        print name

        if attrs.has_key('macAddress'):
            mac = attrs['macAddress'][0]
        else:
            mac = None
        clients.append((name,ip,mac,network_name))

    os.chdir(dnsdir)
    write_zonefiles(clients,zones,dnsprimary,hidden_domains)
    write_rev_zonefile("db."+dnsrevzone,clients,dnsrevzone,zones,dnsprimary,zones[0],hidden_domains,dnsrevzone)
    write_dhcp_config(clients)
