#!/usr/bin/python
# encoding: utf-8
# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4
# plugin api
import plugin

import sys
import os
import ConfigParser
import httplib,urllib
import base64
import ssl
import anydbm

configPath="/etc/dassadmin/devicetypes"

def getContainer(dn):
    return dn.split(',')[1].split('=')[1]


def getGroupName(deviceType):
    path = os.path.join(configPath, deviceType + ".properties")
    result = None

    if os.path.isfile(path):
        with open(path,"r") as f:
            for l in f.readlines():
                k,v=l.split('=',1)
                k = k.strip().lower()
                if k == "nacgroup":
                    result = v.strip()
                    break

    return result


def doXMCRequest(url_with_params, webserver, authHash, method = "GET"):
        headers = { "Accept": "application/json;application/xml;text/html;text/plain",
                   "Authorization" : "Basic %s" % authHash}
        # :(
        if hasattr(ssl,"_create_unverified_context"):
            c = httplib.HTTPSConnection(webserver, 8443, context=ssl._create_unverified_context())
        else:
            c = httplib.HTTPSConnection(webserver, 8443)

        c.request(method, url_with_params, "", headers)
        response = c.getresponse()

        cookie = response.getheader("Set-Cookie", None)

        if response.status==302:
                data = response.getheader('Location')
        else:
                data = response.read()
        c.close()
        return (response.status,data)


def doAddRequest(mac, hostname, group, webserver, authHash, logger=None):
        url = "/axis/services/NACEndSystemWebService/addMACsToEndSystemGroup"

        url_with_params =  \
            "%s?endSystemGroup=%s&macs=%s&descriptions=%s&reauthorize=true&removeFromOtherGroups=true" % \
            (url, urllib.pathname2url(group), mac, hostname)

        if logger is not None:
            logger.log("doAddRequest: " + url_with_params)

        return doXMCRequest(url_with_params, webserver, authHash)


def doDeleteRequest2(mac, webserver, authHash, logger=None):
        url = urllib.pathname2url("/connect/rest/services/endsystem/%sX" % mac)

        if logger is not None:
            logger.log("doDeleteRequest: " + url)

        return doXMCRequest(url, webserver, authHash, "DELETE")


def doDeleteRequest(mac, webserver, authHash, logger=None):
        return doDeleteRequest2(mac, webserver, authHash, logger)
        # below is dead code
        url = "/axis/services/NACEndSystemWebService/deleteEndSystemByMac"

        url_with_params =  \
            "%s?mac=%s&deleteOptionsMask=16" % \
            (url, mac)

        if logger is not None:
            logger.log("doDeleteRequest: " + url_with_params)

        return doXMCRequest(url_with_params, webserver, authHash)


def doDeleteInfoRequest(mac, webserver, authHash):
            url = "/axis/services/NACEndSystemWebService/deleteEndSystemInfoByMac"

            url_with_params =  \
                 "%s?mac=%s" % \
                 (url, mac)

            return doXMCRequest(url_with_params, webserver, authHash)





class XMCAPIPlugin(plugin.Plugin):
        def configure(self,configuration):
                section="XMCAPI"
                self.baseDn=configuration.get(section,"computersDn").lower()
                self.baseDn2=configuration.get(section,"devicesDn").lower()
                self.xmcserver=configuration.get(section,"xmcserver")
                self.username=configuration.get(section,"username")
                self.password=configuration.get(section,"password")
                self.defaultNACGroup=configuration.get(section,"defaultNACGroup")
                self.hostCacheFile=configuration.get(section,"hostCacheFile")
                self.authHash = base64.encodestring('%s:%s' % (self.username, self.password))[:-1]
                self.defaultContainers = [ self.baseDn.split(',')[0].split('=')[1] ]
                # print self.defaultContainers


        def create(self,dn,attrs):
                if not self.filter(dn,None):
                        return
                self.update(dn,attrs)


        def update(self,dn,attrs):
                if not self.filter(dn,attrs):
                        return

                if "macAddress" not in attrs:
                    return

                macAddress = attrs["macAddress"][0]
                hostname = attrs["cn"][0]
                try:
                    container = getContainer(dn)
                except:
                    self.log("failed to determine container for " + dn)
                    return
                # print container, self.defaultContainers

                if container.lower() in self.defaultContainers:
                    group = self.defaultNACGroup
                else:
                    group = getGroupName(container)

                if group is None:
                    self.log("no group defined for %s %s, skipping" % (hostname,container))
                    return

                self.log("setting group to "+group + " for " + hostname + " " + macAddress)

                db = anydbm.open(self.hostCacheFile,"r")
                if hostname in db:
                    oldMac = db[hostname]
                    if macAddress != oldMac:
                        result = doDeleteRequest(oldMac, self.xmcserver, self.authHash, self)
                        self.log("update: MAC address change detected, deleting old entry %s, result: %s" % (oldMac, result[1]))
                db.close()


                result = doAddRequest(macAddress, hostname, group, self.xmcserver, self.authHash, self)
                self.log("response from xmc:"+ str(result[0]) + " " + result[1])

                self.saveHost(hostname, macAddress)

                if result[0] > 399:
                    raise Exception("add request failed (status %d)" % result[0])


        def saveHost(self, hostname, macAddress):
                db = anydbm.open(self.hostCacheFile,"c")
                db[hostname] = macAddress
                db.close()



        def delete(self,dn):
                if not self.filter(dn,None):
                        return


                name = dn.split(",",1)[0].split('=',1)[1]
                mac = None

                db = anydbm.open(self.hostCacheFile,"w")
                if name in db:
                    mac = db[name]
                    del db[name]
                db.close()

                if mac is None:
                    self.log("delete: host %s not found in cache, skipping" % (name))
                    return

                self.log("deleting %s mac %s" % (name, mac))

                result = doDeleteRequest(mac, self.xmcserver, self.authHash, self)
                self.log("delete XMC response: %s %s " % (str(result[0]), result[1]))

                #result = doDeleteInfoRequest(mac, self.xmcserver, self.authHash)
                #self.log("deleting endsystem info %s mac %s, response: %s %s " % (name, mac, str(result[0]), result[1]))


        def rename(self,dn,attrs,dn2):
                if not self.filter(dn,None):
                        return
                self.update(dn2,attrs)


        def exists(self,dn,attrs):
                return False


        def filter(self,dn,attrs):
                lowerdn = dn.lower()
                if lowerdn.endswith(self.baseDn) or lowerdn.endswith(self.baseDn2):
                        return True
                else:
                        return False


if __name__ == "__main__":
        dn = sys.argv[1]
        hostname = sys.argv[2]
        macAddress = sys.argv[3]

        # configPath="/tmp/devicetypes"

        configuration = ConfigParser.ConfigParser()
        configuration.read('/etc/dassadmin/ldapsync.conf')

        p = XMCAPIPlugin()
        p.configure(configuration)
        
        attrs = { "cn":[hostname], "macAddress": [macAddress] }
        
        # p.update(dn, attrs)
        p.saveHost(hostname, macAddress)
        p.delete(dn)
