package de.dassit.dassadmin;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.security.KeyManagementException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.UnrecoverableKeyException;
import java.security.cert.CertificateException;
import java.util.Map;
import java.util.logging.Logger;

import javax.net.ssl.KeyManager;
import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.SSLContext;

import de.dassit.ldap.Connection;
import de.dassit.ldap.QueueLogger;

public class ConnectionTool {
	protected String baseDn;

	protected String bindDn;

	protected String bindPassword;

	protected String ldapUri;

	private static final Logger log = Logger.getLogger(ConnectionTool.class
			.getName());
	
	public String getBaseDn() {
		return baseDn;
	}

	public void setBaseDn(String baseDn) {
		log.info("baseDn set to " + baseDn);
		this.baseDn = baseDn;
	}

	public String getBindDn() {
		return bindDn;
	}

	public void setBindDn(String bindDn) {
		this.bindDn = bindDn;
	}

	public String getBindPassword() {
		return bindPassword;
	}

	public void setBindPassword(String bindPassword) {
		this.bindPassword = bindPassword;
	}

	public String getLdapUri() {
		return ldapUri;
	}

	public void setLdapUri(String ldapUri) {
		this.ldapUri = ldapUri;
	}

	@SuppressWarnings("rawtypes")
	public void configure(Map m) {
		Connection c = new Connection();
		c.setLdapUri(ldapUri);
		c.setBaseDn(baseDn);
		c.setBindDn(bindDn);
		c.setBindPassword(bindPassword);
		if (m.containsKey("keystore")) {
			if (m.containsKey("keystore-password")) {
				createSSLContext((String) m.get("keystore"), (String) m
						.get("keystore-password"));
			} else {
				log.severe("need keystore-password");
			}
		}
		log.info("Initializing connection " + ldapUri + " " + baseDn);
		c.setChangeLogger(new QueueLogger());
		try {
			c.connect();
		} catch (IOException e) {
			log.warning("Connection failed: " + e.getMessage());
		}
		Connection.setDefaultConnection(c);
	}

	public void createSSLContext(String keyStore, String keyStorePassword) {
		SSLContext sslCtx;

		try {
			String alg = KeyManagerFactory.getDefaultAlgorithm();
			KeyManagerFactory kmf;

			kmf = KeyManagerFactory.getInstance(alg);

			FileInputStream f = new FileInputStream(keyStore);
			KeyStore ks = KeyStore.getInstance("jks");
			ks.load(f, keyStorePassword.toCharArray());
			f.close();

			kmf.init(ks, keyStorePassword.toCharArray());

			KeyManager[] kms = kmf.getKeyManagers();

			sslCtx = SSLContext.getInstance("SSL");
			sslCtx.init(kms, null, null);
			SSLContext.setDefault(sslCtx);
		} catch (NoSuchAlgorithmException e) {
			e.printStackTrace();
		} catch (KeyManagementException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (KeyStoreException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (UnrecoverableKeyException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (CertificateException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
}
