package de.dassit.dassadmin;

import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.logging.Logger;

import javax.naming.NamingException;
import javax.naming.directory.SearchResult;

import de.dassit.ldap.Connection;
import de.dassit.ldap.Entry;
import de.dassit.ldap.EntryList;
import de.dassit.ldap.Group;

public class DassGroup extends Group {
	private static final Logger log = Logger.getLogger(DassGroup.class
			.getName());
	
	static String templateContainer = "ou=dassadmin";

	static String extraObjectClasses;

	static boolean useSambaAttrs = false;

	public DassGroup() {
		super();
	}

	public DassGroup(SearchResult arg0) {
		super(arg0);
	}

	public DassGroup(String arg0, int arg1) {
		super(arg0, arg1);
		// TODO Auto-generated constructor stub
	}

	@Override
	protected EntryList<? extends Entry> getEntryList() {
		return new EntryList<DassGroup>() {
			public void addInstance(SearchResult sr) {
				entries.add(new DassGroup(sr));
			}
		};
	}

	@Override
	protected void setNamingParameters() {
		setNamingAttribute(defaultNamingAttribute);
		setObjectClasses(new String[] { "top", "posixGroup" });
		addObjectClassesCSV(extraObjectClasses);
		setContainer(defaultContainer);
	}

	public void setDefaultContainer(String ou) {
		super.setDefaultContainer(ou);
		Group factory = new Group();
		factory.setDefaultContainer(ou);
	}

	public void setTemplateContainer(String ou) {
		templateContainer = ou;
	}

	public void setExtraObjectClasses(String csv) {
		extraObjectClasses = csv;
	}

	public void setSambaSidFromGid() throws Exception {
		String sid = getAttribute("sambaSID");
		int position = sid.lastIndexOf('-');
		if (position < 0)
			throw new Exception(getName() + " has invalid sambaSID");
		int rid = getGid() * 2 + 1001;
		sid = sid.substring(0, position + 1) + rid;
		setAttribute("sambaSID", sid);
	}

	public List<DassGroup> getAll() {
		return searchDassGroups("objectClass=posixGroup", defaultContainer);
	}

	public static DassGroup fetchById(int gid) {
		final String filter = "gidNumber=" + gid;
		List<DassGroup> result = searchDassGroups(filter,
				getDefaultContainer(DassGroup.class.getName()));
		if (result.size() > 0)
			return result.get(0);
		else
			return null;
	}

	public static List<DassGroup> searchDassByMemberName(String name) {
		final String filter = "(&(objectClass=posixGroup)(memberUid=" + name
				+ "))";
		return searchDassGroups(filter,
				getDefaultContainer(DassGroup.class.getName()));
	}

	public static List<DassGroup> searchDassGroups(String filter,
			String container) {

		EntryList<DassGroup> entryList = new EntryList<DassGroup>() {
			public void addInstance(SearchResult sr) {
				entries.add(new DassGroup(sr));
			}
		};

		Connection.getDefaultConnection().search(filter, container, entryList);
		List<DassGroup> result = entryList.getEntries();

		Collections.sort(result, new Comparator<DassGroup>() {

			public int compare(DassGroup arg0, DassGroup arg1) {
				return arg0.getName().compareToIgnoreCase(arg1.getName());
			}
		});

		return entryList.getEntries();
	}

	public static int getNextGid() throws NamingException {
		List<DassGroup> result = DassGroup.searchDassGroups("cn=templateGroup",
				templateContainer);
		DassGroup template = result.get(0);

		int nextGid = template.getGid();

		log.info("next gidNumber " + nextGid);
		DassGroup factory = new DassGroup();
		while (factory.search("gidNumber=" + nextGid,
				getDefaultContainer(factory.getClass().getName())).size() != 0) {
			nextGid++;
			log.info("trying gidNumber " + nextGid);
		}
		log.info(nextGid + " ok");
		template.setGid(nextGid + 1);
		template.save();
		return nextGid;
	}

	public static DassGroup getTemplateGroup() {
		DassGroup g = searchDassGroups("cn=templateGroup", templateContainer)
				.get(0);
		g.setContainer(getDefaultContainer(DassGroup.class.getName()));
		g.saveAsNew();
		return g;
	}

}
