package de.dassit.dassadmin;

import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import javax.naming.directory.SearchResult;

import de.dassit.ldap.Connection;
import de.dassit.ldap.Entry;
import de.dassit.ldap.EntryList;

public class DassMailAlias extends Entry {
	static String recipientAttribute = "suseMailAcceptAddress";
	static String destinationsAttribute = "suseMailForwardAddress";
	static String extraObjectClasses;
	static String someObjectClass;

	public DassMailAlias() {
		super();
	}

	public DassMailAlias(SearchResult sr) {
		super(sr);
	}

	@Override
	protected EntryList<? extends Entry> getEntryList() {
		return new EntryList<DassMailAlias>() {
			public void addInstance(SearchResult sr) {
				entries.add(new DassMailAlias(sr));
			}
		};
	}

	@Override
	protected void setNamingParameters() {
		setNamingAttribute(defaultNamingAttribute);
		setObjectClasses(new String[0]);
		addObjectClassesCSV(extraObjectClasses);
		setContainer(defaultContainer);
	}

	public void setExtraObjectClasses(String csv) {
		extraObjectClasses = csv;
		someObjectClass = extraObjectClasses.split(",")[0];
	}

	public void setRecipientAttribute(String s) {
		recipientAttribute = s;
	}

	public void setDestinationsAttribute(String s) {
		destinationsAttribute = s;
	}

	public String getRecipient() {
		return getAttribute(recipientAttribute);
	}

	public void setRecipient(String s) {
		setAttribute(recipientAttribute, s);
	}

	public List<String> getDestinations() {
		List<String> result = getAttributeMultiValue(destinationsAttribute);
		if (result == null)
			return null;
		Collections.sort(result);
		return result;
	}

	public void addDestination(String s) {
		appendAttributeValue(destinationsAttribute, s);
	}

	public void removeDestination(String s) {
		removeAttributeValue(destinationsAttribute, s);
	}

	public List<DassMailAlias> getAll() {
		return search("objectClass=" + someObjectClass, defaultContainer);
	}

	public List<DassMailAlias> search(String filter, String container) {

		EntryList<DassMailAlias> entryList = new EntryList<DassMailAlias>() {
			public void addInstance(SearchResult sr) {
				entries.add(new DassMailAlias(sr));
			}
		};

		Connection.getDefaultConnection().search(filter, container, entryList);
		List<DassMailAlias> result = entryList.getEntries();

		Collections.sort(result, new Comparator<DassMailAlias>() {

			public int compare(DassMailAlias arg0, DassMailAlias arg1) {
				return arg0.getName().compareToIgnoreCase(arg1.getName());
			}
		});

		return entryList.getEntries();
	}

}
