package de.dassit.dassadmin;

import java.util.List;

import javax.naming.NamingException;
import javax.naming.directory.SearchResult;

import de.dassit.ldap.Connection;
import de.dassit.ldap.Entry;
import de.dassit.ldap.EntryList;

public class IpDevice extends Entry {
	static String extraObjectClasses;
	static String networkNameAttribute = "ou";

	public IpDevice() {
		super();
	}

	public IpDevice(SearchResult sr) {
		super(sr);
	}

	@Override
	protected EntryList<? extends Entry> getEntryList() {
		return new EntryList<IpDevice>() {
			public void addInstance(SearchResult sr) {
				entries.add(new IpDevice(sr));
			}
		};
	}

	@Override
	public void save() throws NamingException {
		String ipHostNumber = get("ipHostNumber");
		if (ipHostNumber != null && ipHostNumber.equals("auto")) {
			Network factory = new Network();
			Network net = (Network) factory.fetch(getNetworkName());
			put("ipHostNumber", net.getNextIpAddress());
		}
		super.save();
	}

	@Override
	protected void setNamingParameters() {
		setNamingAttribute(defaultNamingAttribute);
		setObjectClasses(new String[] { "top", "device", "ipHost", "ieee802Device" });
		addObjectClassesCSV(extraObjectClasses);
		setContainer(defaultContainer);
	}

	public Network getNetwork() {
		Network factory = new Network();
		return (Network) factory.fetch(get("ou"));
	}

	public void setNetworkName(String s) {
		put(networkNameAttribute, s);
	}

	public String getNetworkName() {
		return get(networkNameAttribute);
	}

	public String getDeviceTypeName() {
		String rdn = getRDN();
		int start = rdn.indexOf('=') + 1;
		start = rdn.indexOf('=', start) + 1;
		int end = rdn.indexOf(',', start);
		if(end<0) end = rdn.length();
		return rdn.substring(start, end);
	}

	public static List<IpDevice> searchIpDevices(String filter) {
		return searchIpDevices(filter, getDefaultContainer(IpDevice.class.getName()));
	}

	public static List<IpDevice> searchIpDevices(String filter, String container) {

		EntryList<IpDevice> entryList = new EntryList<IpDevice>() {
			public void addInstance(SearchResult sr) {
				entries.add(new IpDevice(sr));
			}
		};

		Connection.getDefaultConnection().search(filter, container, entryList);

		return entryList.getEntries();
	}

	@Override
	public List<IpDevice> search(String filter, String container) {
		return IpDevice.searchIpDevices(filter, container);
	}

	@Override
	public List<IpDevice> search(String filter) {
		return IpDevice.searchIpDevices(filter, defaultContainer);
	}

	public List<IpDevice> getAll() {
		return IpDevice.searchIpDevices("(&(objectClass=ipHost)(objectClass=device))", defaultContainer);
	}

	public static String getExtraObjectClasses() {
		return extraObjectClasses;
	}

	public void setExtraObjectClasses(String extraObjectClasses) {
		IpDevice.extraObjectClasses = extraObjectClasses;
	}

	public static String getNetworkNameAttribute() {
		return networkNameAttribute;
	}

	public static void setNetworkNameAttribute(String networkNameAttribute) {
		IpDevice.networkNameAttribute = networkNameAttribute;
	}

	public static IpDevice searchByIpAddress(String address) {
		List<IpDevice> result = searchIpDevices("ipHostNumber=" + address);
		if (result.size() < 1)
			return null;
		return (IpDevice) result.get(0);
	}
}
