package de.dassit.dassadmin;

import java.io.UnsupportedEncodingException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.naming.directory.SearchResult;

import de.dassit.ldap.Entry;
import de.dassit.ldap.EntryList;
import de.dassit.ldap.User;

public class SmbUser extends User {

	static SimpleDateFormat dateParser = new SimpleDateFormat("dd.MM.yyyy");

	public SmbUser() {
		super();
	}

	public SmbUser(SearchResult sr) {
		super(sr);
	}

	public SmbUser(String uid, int uidNumber, int gidNumber, String cn,
			String sn, String home) {
		super(uid, uidNumber, gidNumber, cn, sn, home);
	}

	@Override
	protected EntryList<? extends Entry> getEntryList() {
		return new EntryList<SmbUser>() {
			public void addInstance(SearchResult sr) {
				entries.add(new SmbUser(sr));
			}
		};
	}

	@Override
	public void encryptAndSetPassword(String s) {
		super.encryptAndSetPassword(s);
		setSmbPassword(s);
	}

	public void setSmbPassword(String s) {
		// create MD4 digest for NT password

		Md4 m = new Md4();
		try {
			m.reset();
			byte[] b = s.getBytes("UTF-16LE");
			m.update(b, 0, b.length);
		} catch (UnsupportedEncodingException e) {
			// ignore
			e.printStackTrace();
		}

		StringBuffer ntHash = new StringBuffer();
		byte[] hash = m.digest();
		for (int i = 0; i < hash.length; i++) {
			ntHash.append(String.format("%02X", hash[i]));
		}

		setAttribute("sambaNTPassword", ntHash.toString());

		Long now = new Long(new Date().getTime() / 1000);
		setAttribute("sambaPwdLastSet", now.toString());
	}

	public void setSambaSidFromUid() throws Exception {
		String sid = getAttribute("sambaSID");
		if (sid == null)
			throw new Exception(getName() + " has no sambaSID");

		int position = sid.lastIndexOf('-');
		if (position < 0)
			throw new Exception(getName() + " has invalid sambaSID");
		int rid = getUid() * 2 + 1000;
		sid = sid.substring(0, position + 1) + rid;
		setAttribute("sambaSID", sid);
	}

	public String getSambaKickOffTime() {
		String s = getAttribute("sambaKickOffTime");
		if (s == null)
			return null;

		// convert from milliseconds to seconds
		Date d = new Date(Long.parseLong(s) * 1000000);
		return dateParser.format(d);
	}

	public void setSambaKickOffTime(String s) {
		if (s == null || s.equals(""))
			return;
		Date d;
		String timeAsString = null;
		try {
			d = dateParser.parse(s);
			timeAsString = Long.toString((d.getTime() / 1000000) + 1);
		} catch (ParseException e) {
			// should not happen
			e.printStackTrace();
		}
		setAttribute("sambaKickOffTime", timeAsString);
	}

	public boolean getSambaAcctFlag(String flag) {
		String acctFlags = getAttribute("sambaAcctFlags");
		if (acctFlags == null)
			return false;
		return (acctFlags.indexOf(flag) >= 0);
	}

	public void setSambaAcctFlag(String flag, boolean value) {
		char c = ' ';
		String af = getAttribute("sambaAcctFlags");
		if (af == null)
			af = "[]";

		StringBuffer acctFlags = new StringBuffer(af);

		if (value)
			c = flag.charAt(0);

		int i = acctFlags.indexOf(flag);

		if (!value && i < 0)
			return;

		if (i < 0) {
			i = acctFlags.indexOf(" ");
			if (i < 0)
				acctFlags.insert(1, c);
			else
				acctFlags.setCharAt(i, c);
		} else {
			acctFlags.setCharAt(i, c);
		}

		setAttribute("sambaAcctFlags", acctFlags.toString());
	}

}