package de.dassit.ldap;

import java.util.ArrayList;
import java.util.List;

import javax.naming.directory.SearchResult;

public class Group extends Entry {
	@Override
	protected EntryList<? extends Entry> getEntryList() {
		return new EntryList<Group>() {
			public void addInstance(SearchResult sr) {
				entries.add(new Group(sr));
			}
		};
	}

	public Group() {
	}

	public Group(SearchResult sr) {
		super(sr);
	}

	public Group(String name, int gidNumber) {
		super();
		setNamingParameters();
		setAttribute("cn", name);
		setAttribute("gidNumber", Integer.toString(gidNumber));
	}

	protected void setNamingParameters() {
		setNamingAttribute("cn");
		setObjectClasses(new String[] { "top", "posixGroup" });
		setContainer(defaultContainer);
	}

	public String getName() {
		return getAttribute("cn");
	}

	public void setName(String s) {
		put("cn", s);
	}

	public int getGid() {
		return Integer.parseInt(getAttribute("gidNumber"));
	}

	public String getDescription() {
		return getAttribute("description");
	}

	public void setDescription(String s) {
		put("description", s);
	}

	public List<String> getMemberNames() {
		int count = getAttributeSize("memberUid");
		List<String> result = new ArrayList<String>(count);
		for (int i = 0; i < count; i++) {
			result.add(getAttribute("memberUid", i));
		}
		return result;
	}

	public void addMemberName(String username) {
		appendAttributeValue("memberUid", username);
	}

	public void removeMemberName(String username) {
		removeAttributeValue("memberUid", username);
	}

	public boolean hasMember(User u) {
		List<String> memberNames = getMemberNames();
		for (String n : memberNames) {
			if (n.equals(u.getName()))
				return true;
		}
		return false;
	}

	public void setGid(int gid) {
		put("gidNumber", Integer.toString(gid));
	}

	public List<? extends Entry> searchByMemberName(String name) {
		final String filter = "(&(objectClass=posixGroup)(memberUid=" + name
				+ "))";
		return search(filter);
	}

	public List<? extends Entry> getAll() {
		return search("objectClass=posixGroup");
	}

	@Override
	public boolean isMultivalue(String attributeName) {
		if (attributeName.equals("member") || attributeName.equals("memberUid"))
			return true;
		return super.isMultivalue(attributeName);
	}

	@Override
	public boolean isInteger(String attrName) {
		if (attrName.equals("gidNumber"))
			return true;

		return super.isInteger(attrName);
	}
}
