package de.dassit.ldap;

import java.util.List;

import javax.naming.directory.SearchResult;

public class GroupOfNames extends Entry {
	@Override
	protected EntryList<? extends Entry> getEntryList() {
		return new EntryList<GroupOfNames>() {
			public void addInstance(SearchResult sr) {
				entries.add(new GroupOfNames(sr));
			}
		};
	}

	public GroupOfNames() {
		super();
	}

	public GroupOfNames(SearchResult sr) {
		super(sr);
	}

	protected void setNamingParameters() {
		setNamingAttribute("cn");
		setObjectClasses(new String[] { "top", "groupOfNames" });
		setContainer(defaultContainer);
	}

	public String getName() {
		return getAttribute("cn");
	}

	public void setName(String s) {
		put("cn", s);
	}

	public void addMember(Entry e) {
		appendAttributeValue("member", e.getDN());
	}

	public void removeMember(Entry e) {
		removeAttributeValue("member", e.getDN());
	}

	public List<String> getMemberDNs() {
		return getAttributeMultiValue("member");
	}

	public boolean hasMember(Entry e) {
		List<String> members = getMemberDNs();
		for (String n : members) {
			if (n.equals(e.getDN()))
				return true;
		}
		return false;
	}

	public List<? extends Entry> searchByMember(Entry e) {
		final String filter = "(&(objectClass=groupOfNames)(member="
				+ e.getDN() + "))";
		return search(filter);
	}

	public List<? extends Entry> getAll() {
		return search("objectClass=groupOfNames");
	}
}
