package de.dassit.ldap;

import java.io.IOException;
import java.security.MessageDigest;
import java.util.Random;

import de.dassit.dassadmin.Base64;

public class SSHAPassword {
	protected byte[] hashBytes;
	protected byte[] salt;
	protected String crypted;
	
	public final static int hashSize = 20;
	public final static int saltSize = 4;

	public SSHAPassword() {
		
	}

	public SSHAPassword(String h) throws IOException {
		byte[] buffer;

		if(h.startsWith("{SSHA}")) {
			h=h.substring(6);
		}
		buffer=Base64.decode(h);
		if(buffer.length!=24) throw new IOException("invalid SHA1 hash");
		hashBytes= new byte[20];
		System.arraycopy(buffer,0,hashBytes,0,hashSize);
		salt = new byte[saltSize];
		System.arraycopy(buffer,hashSize,salt,0,saltSize);
	}
	
	public String getEncrypted() {
		return crypted;
	}

	public void encryptAndSet(String cleartext) {
		MessageDigest sha;

		try {
			sha=MessageDigest.getInstance("SHA");
		}
		catch(java.security.NoSuchAlgorithmException ex) {
			return;
		}

		sha.reset();
		try {
			sha.update(cleartext.getBytes("UTF-8"));
		}
		catch (java.io.UnsupportedEncodingException ex) {
			sha.update(cleartext.getBytes());
		}

		salt=new byte[saltSize];
		Random random=new Random();
		random.nextBytes(salt);

		sha.update(salt);	

		hashBytes=sha.digest();

		byte[] buffer=new byte[hashSize+saltSize];
		System.arraycopy(hashBytes,0,buffer,0,hashSize);		
		System.arraycopy(salt,0,buffer,hashSize,saltSize);
		
		crypted="{SSHA}"+Base64.encodeBytes(buffer);
	}
	
	public boolean verify(String password) {

		MessageDigest sha;

		try {
			sha=MessageDigest.getInstance("SHA");
		}
		catch(java.security.NoSuchAlgorithmException ex) {
			return false;
		}

		sha.reset();
		try {
			sha.update(password.getBytes("UTF-8"));
		}
		catch (java.io.UnsupportedEncodingException ex) {
			sha.update(password.getBytes());
		}

		sha.update(salt);
		
		byte[] digest=sha.digest();

		if(MessageDigest.isEqual(hashBytes,digest))
			return true;
		
		return false;
	}

	public static void test(String[] argv) throws IOException {
		SSHAPassword p=new SSHAPassword();
		p.encryptAndSet("passwort");
		System.out.println(p.crypted);
		if(p.verify("passwort")) System.out.println("Password ok");
	}
}
