/*
 * Copyright 2010 Sebastian Lederer <slederer@insignificance.de>
 */

package de.dassit.ldap;

import java.util.ArrayList;
import java.util.List;

import javax.naming.NamingException;
import javax.naming.directory.SearchResult;

public class User extends Entry {
	public User() {
		super();
	}

	public User(SearchResult sr) {
		super(sr);
	}

	protected void setNamingParameters() {
		setNamingAttribute(defaultNamingAttribute);
		setObjectClasses(new String[] { "top", "person", "inetOrgPerson",
				"posixAccount" });
		setContainer(defaultContainer);
	}

	public User(String uid, int uidNumber, int gidNumber, String cn, String sn,
			String home) {
		super();
		setNamingParameters();
		setName(uid);
		setUid(uidNumber);
		setGid(gidNumber);
		setAttribute("cn", cn);
		setAttribute("sn", sn);
		setAttribute("homeDirectory", home);
	}

	public String getUsername() {
		return getAttribute("uid");
	}

	public int getUid() {
		return getUidNumber();
	}

	public int getGid() {
		return getGidNumber();
	}

	public int getUidNumber() {
		return Integer.parseInt(getAttribute("uidNumber").toString());
	}

	public int getGidNumber() {
		return Integer.parseInt(getAttribute("gidNumber").toString());
	}

	public ArrayList<String> getMail() {
		return getAttributeMultiValue("mail");
	}

	public void addMailValue(String mail) {
		appendAttributeValue("mail", mail);
	}

	public void removeMailValue(String mail) {
		removeAttributeValue("mail", mail);
	}

	public String getDescription() {
		return getAttribute("description");
	}

	public void setDescription(String s) {
		put("description", s);
	}

	public String getDisplayName() {
		return getAttribute("displayName");
	}

	public void setDisplayName(String s) {
		put("displayName", s);
	}

	public String getGivenName() {
		return getAttribute("givenName");
	}

	public void setGivenName(String s) {
		put("givenName", s);
	}

	public String getSurname() {
		return getAttribute("sn");
	}

	public void setSurname(String s) {
		put("sn", s);
	}

	public String getHomeDirectory() {
		return getAttribute("homeDirectory");
	}

	public void setHomeDirectory(String s) {
		put("homeDirectory", s);
	}

	public String getLoginShell() {
		return getAttribute("loginShell");
	}

	public void setLoginShell(String s) {
		put("loginShell", s);
	}

	public String getUserPassword() {
		return getAttribute("userPassword");
	}

	public void setUserPassword(String s) {
		put("userPassword", s);
	}

	public String getTelephoneNumber() {
		return getAttribute("telephoneNumber");
	}

	public void setTelephoneNumber(String s) {
		put("telephoneNumber", s);
	}

	public String getRoomNumber() {
		return getAttribute("roomNumber");
	}

	public void setRoomNumber(String s) {
		put("roomNumber", s);
	}

	public void setGid(int gid) {
		setGidNumber(gid);
	}

	public void setUid(int uid) {
		setUidNumber(uid);
	}

	public void setGidNumber(int gid) {
		put("gidNumber", Integer.toString(gid));
	}

	public void setUidNumber(int uid) {
		put("uidNumber", Integer.toString(uid));
	}

	public List<? extends Entry> getGroups() {
		Group factory = new Group();
		return factory.searchByMemberName(getName());
	}

	public void removeFromGroups() throws NamingException {
		for (Entry e : getGroups()) {
			Group g = (Group) e;
			g.removeMemberName(getName());
			g.save();
		}
	}

	@Override
	public void delete() throws NamingException {
		removeFromGroups();
		super.delete();
	}

	@Override
	public void rename(String newName) throws NamingException {
		List<? extends Entry> groups = getGroups();

		// change groups only if this object is located in
		// the standard users container
		if(!(getRDN().equals(defaultContainer))) {
			super.rename(newName);
			return;
		}
		
		removeFromGroups();
		super.rename(newName);
		for (Entry e : groups) {
			Group g = (Group) e;
			g.addMemberName(newName);
			g.save();
		}
	}

	public void encryptAndSetPassword(String s) {
		// set SSHA password
		SSHAPassword p = new SSHAPassword();
		p.encryptAndSet(s);
		setAttribute("userPassword", p.getEncrypted());
	}

	@Override
	protected EntryList<? extends Entry> getEntryList() {
		return new EntryList<User>() {
			public void addInstance(SearchResult sr) {
				entries.add(new User(sr));
			}
		};
	}

	@Override
	public boolean isInteger(String attrName) {
		if (attrName.equals("uidNumber") || attrName.equals("gidNumber"))
			return true;

		return super.isInteger(attrName);
	}

	@Override
	public boolean isHidden(String attrName) {
		if (attrName.equals("userPassword"))
			return true;

		return super.isHidden(attrName);
	}
}